@@ -21,8 +21,10 @@ from random import uniform, gauss
 from sequana.lazy import numpy as np
 from sequana.lazy import pylab
 
-from sequana import logger
-logger.name = __name__
+import colorlog
+logger = colorlog.getLogger(__name__)
+
+
 
 
 __all__ = ["MetropolisHasting"]
