@@ -28,10 +28,13 @@ from sequana.utils import config
 
 from sequana.lazy import pandas as pd
 from sequana.lazy import pylab
-from sequana import logger
 
 from sequana.utils.datatables_js import DataTable
 
+import colorlog
+logger = colorlog.getLogger(__name__)
+
+
 
 class CutadaptModule(SequanaBaseModule):
     """ Write HTML report of coverage analysis. This class takes either a
