@@ -43,12 +43,22 @@ import glob
 __all__ = ["RNADiffAnalysis", "RNADiffResults", "RNADiffTable", "RNADesign"]
 
 
+def strip(text):
+    try:
+        return text.strip()
+    except AttributeError:
+        return text
+
 class RNADesign:
     """Simple RNA design handler"""
 
-    def __init__(self, filename, reference=None):
+    def __init__(self, filename, sep="\s*,\s*", reference=None):
         self.filename = filename
-        self.df = pd.read_csv(filename, sep=",")
+        # \s to strip the white spaces
+        self.df = pd.read_csv(filename, sep=sep, engine="python", 
+                comment="#",
+                dtype={"label": str})
+
         self.reference = reference
 
     def _get_conditions(self):
@@ -91,15 +101,16 @@ class RNADiffAnalysis:
     :param annot_cols: GFF attributes to use for results annotations
     :param threads: Number of threads to use
     :param outdir: Path to output directory.
-    :param sep: The separator to use in dataframe exports
+    :param sep_counts: The separator used in the input count file.
+    :param sep_design: The separator used in the input design file.
 
     This class reads a :class:`sequana.featurecounts.`
 
-    r = rnadiff.RNADiffAnalysis("counts.csv", "design.csv",
-            condition="condition", comparisons=[(("A", "B"), ('A', "C")],
-            fc_feature="gene",
-            fc_attribute="ID", gff="mygff.gff")
 
+    ::
+
+        r = rnadiff.RNADiffAnalysis("counts.csv", "design.csv",
+                condition="condition", comparisons=[(("A", "B"), ('A', "C")],
 
     """
 
@@ -126,50 +137,26 @@ class RNADiffAnalysis:
         threads=4,
         outdir="rnadiff",
         sep_counts=",",
-        sep_design=",",
+        sep_design="\s*,\s*",
     ):
 
-        self.counts_filename = counts_file
-        self.design_filename = design_file
+        self.outdir = Path(outdir)
 
-        self.counts = pd.read_csv(
-            counts_file, sep=sep_counts, index_col="Geneid", comment="#"
+        self.usr_counts = counts_file
+        self.usr_design = design_file
+
+        self.counts_filename = self.outdir / "counts.csv"
+        self.design_filename = self.outdir / "design.csv"
+
+        self.counts, self.design = self.check_and_save_input_tables(
+            sep_counts, sep_design
         )
-        self.design = pd.read_csv(
-            design_file,
-            sep=sep_design,
-            comment="#",
-            dtype={"label": str},
-        ).set_index("label")
 
-        # TODO: Check and resorting if necessary
-        if sorted(list(self.counts.columns)) != sorted(list(self.design.index)):
-            logger.error(f"Counts columns and design rows does not match.")
-            logger.error(self.counts.columns)
-            logger.error(self.design.index)
-            sys.exit(1)
-
-        # set condition of the statistical model
-        columns = ",".join(self.design.columns)
-        if condition not in columns:
-            logger.error(
-                f"""Your condition named '{condition}' is expected to
-be in the header of your design file but was not found. Candidates are:
-    {columns}"""
-            )
-            sys.exit(1)
         self.condition = condition
-        self.comparisons = comparisons
+        self.check_condition()
 
-        # let us check the consistenct of the design and comparisons
-        valid_conditions = ",".join(set(self.design[condition].values))
-        for item in [x for y in self.comparisons for x in y]:
-            if item not in self.design[condition].values:
-                logger.error(
-                    f"""{item} not found in the design. Fix the design
-or comparisons. possible values are {valid_conditions}"""
-                )
-                sys.exit(1)
+        self.comparisons = comparisons
+        self.check_comparisons()
 
         self.comparisons_str = (
             f"list({', '.join(['c' + str(x) for x in self.comparisons])})"
@@ -183,10 +170,9 @@ or comparisons. possible values are {valid_conditions}"""
         self.gff = gff
         self.fc_feature = fc_feature
         self.fc_attribute = fc_attribute
+        self.annot_cols = annot_cols
         self.threads = threads
 
-        self.outdir = Path(outdir)
-
     def __repr__(self):
         info = f"RNADiffAnalysis object:\n\
 - {self.counts.shape[1]} samples.\n\
@@ -198,6 +184,56 @@ Design overview:\n\
 
         return info
 
+    def check_and_save_input_tables(self, sep_counts, sep_design):
+        try:
+            self.outdir.mkdir()
+        except:
+            pass
+
+        counts = pd.read_csv(
+            self.usr_counts, sep=sep_counts, index_col="Geneid", comment="#"
+        ).sort_index(axis=1)
+
+
+        design = RNADesign(self.usr_design, sep=sep_design)
+        design = design.df.set_index("label").sort_index()
+
+        if list(counts.columns) != list(design.index):
+            logger.error(
+                f"Counts columns and design rows does not match (after sorting)."
+            )
+            logger.error(counts.columns)
+            logger.error(design.index)
+            sys.exit(1)
+
+        counts.to_csv(self.counts_filename)
+        design.to_csv(self.design_filename)
+
+        return counts, design
+
+    def check_condition(self):
+        # set condition of the statistical model
+
+        columns = ",".join(self.design.columns)
+        if self.condition not in columns:
+            logger.error(
+                f"""Your condition named '{condition}' is expected to
+be in the header of your design file but was not found. Candidates are:
+    {columns}"""
+            )
+            sys.exit(1)
+
+    def check_comparisons(self):
+        # let us check the consistenct of the design and comparisons
+        valid_conditions = ",".join(set(self.design[self.condition].values))
+        for item in [x for y in self.comparisons for x in y]:
+            if item not in self.design[self.condition].values:
+                logger.error(
+                    f"""{item} not found in the design. Fix the design
+or comparisons. possible values are {valid_conditions}"""
+                )
+                sys.exit(1)
+
     def run(self):
         """Create outdir and a DESeq2 script from template for analysis. Then execute
         this script.
@@ -205,39 +241,52 @@ Design overview:\n\
         :return: a :class:`RNADiffResults` instance
         """
         logger.info("Running DESeq2 analysis. Please wait")
-        try:
-            self.outdir.mkdir()
-        except:
-            pass
 
-        with open("rnadiff_light.R", "w") as f:
+        rnadiff_script = self.outdir / "rnadiff_light.R"
+
+        with open(rnadiff_script, "w") as f:
             f.write(RNADiffAnalysis.template.render(self.__dict__))
 
         logger.info("Starting differential analysis with DESeq2...")
 
-        p = subprocess.run(
-            ["Rscript", "rnadiff_light.R"], universal_newlines=True, capture_output=True
-        )
-
+        # capture_output is valid for py3.7 and above so we will use
+        # stdout/stderr to be back compatible with py3.6
         # Capture rnadiff output
-        with open("rnadiff.err", "w") as f:
-            f.write(p.stderr)
-        with open("rnadiff.out", "w") as f:
-            f.write(p.stdout)
+        p = subprocess.Popen(
+            f"Rscript {rnadiff_script}",
+            shell=True,
+            universal_newlines=True,
+            stderr=subprocess.PIPE,
+            stdout=subprocess.PIPE,
+        )
+        p.wait()
+        stdout, stderr = p.stdout.read(), p.stderr.read()
+
+        # Capture rnadiff output, Unfortunately, R code mixes stdout/stderr
+        # FIXME
+        with open(self.outdir / "rnadiff.err", "w") as f:
+            f.write(stderr)
+        with open(self.outdir / "rnadiff.out", "w") as f:
+            f.write(stdout)
+
+        # if os.path.exists():
+        #    logger.error(f"stderr of R call is not empty: {p.stderr}")
+        #    sys.exit(1)
 
         results = RNADiffResults(
             self.outdir,
             self.design_filename,
-            group=self.condition,
+            condition=self.condition,
             gff=self.gff,
             fc_feature=self.fc_feature,
             fc_attribute=self.fc_attribute,
+            annot_cols=self.annot_cols,
         )
         return results
 
 
 class RNADiffTable:
-    def __init__(self, path, alpha=0.05, log2_fc=0, sep=","):
+    def __init__(self, path, alpha=0.05, log2_fc=0, sep=",", condition="condition"):
         """ A representation of the results of a single rnadiff comparison """
         self.path = Path(path)
         self.name = self.path.stem.replace("_degs_DESeq2", "").replace("-", "_")
@@ -246,6 +295,7 @@ class RNADiffTable:
         self._log2_fc = log2_fc
 
         self.df = pd.read_csv(self.path, index_col=0, sep=sep)
+        self.condition = condition
 
         self.filt_df = self.filter()
         self.set_gene_lists()
@@ -273,17 +323,22 @@ class RNADiffTable:
     def filter(self):
         """filter a DESeq2 result with FDR and logFC thresholds"""
 
-        fc_filt = self.df["log2FoldChange"].abs() >= self._log2_fc
-        fdr_filt = self.df["padj"] <= self._alpha
+        fc_filt = self.df["log2FoldChange"].abs() < self._log2_fc
+        fdr_filt = self.df["padj"] > self._alpha
+        outliers = self.df["padj"].isna()
 
-        return self.df[fc_filt.values & fdr_filt.values]
+        filt_df = self.df.copy()
+        filt_df[fc_filt.values | fdr_filt.values | outliers] = np.NaN
+        return filt_df
 
     def set_gene_lists(self):
 
+        only_drgs_df = self.filt_df.dropna(how="all")
+
         self.gene_lists = {
-            "up": list(self.filt_df.query("log2FoldChange > 0").index),
-            "down": list(self.filt_df.query("log2FoldChange < 0").index),
-            "all": list(self.filt_df.index),
+            "up": list(only_drgs_df.query("log2FoldChange > 0").index),
+            "down": list(only_drgs_df.query("log2FoldChange < 0").index),
+            "all": list(only_drgs_df.index),
         }
 
     def summary(self):
@@ -306,6 +361,7 @@ class RNADiffTable:
         markersize=4,
         limit_broken_line=[20, 40],
         plotly=False,
+        annotations=None,
     ):
         """
 
@@ -319,25 +375,31 @@ class RNADiffTable:
             r.plot_volcano()
 
         """
-        d1 = self.df.query("padj>@padj")
-        d2 = self.df.query("padj<=@padj")
 
         if plotly:
             from plotly import express as px
 
             df = self.df.copy()
-            df["log_adj_pvalue"] = -pylab.log10(self.df.padj)
+
+            if annotations is not None:
+                try:
+                    df = pd.concat([df, annotations.annotation], axis=1)
+                except Exception as err:
+                    logger.warning(
+                        f"Could not merge rnadiff table with annotation. Full error is: {err}"
+                    )
+            df["log_adj_pvalue"] = -pylab.log10(df.padj)
             df["significance"] = [
                 "<{}".format(padj) if x else ">={}".format(padj) for x in df.padj < padj
             ]
 
-            if "Name" in self.df.columns:
+            if "Name" in df.columns:
                 hover_name = "Name"
-            elif "gene_id" in self.df.columns:
+            elif "gene_id" in df.columns:
                 hover_name = "gene_id"
-            elif "locus_tag" in self.df.columns:
+            elif "locus_tag" in df.columns:
                 hover_name = "locus_tag"
-            elif "ID" in self.df.columns:
+            elif "ID" in df.columns:
                 hover_name = "ID"
             else:
                 hover_name = None
@@ -388,6 +450,8 @@ class RNADiffTable:
         else:
             bax = pylab
 
+        d1 = self.df.query("padj>@padj")
+        d2 = self.df.query("padj<=@padj")
         bax.plot(
             d1.log2FoldChange,
             -np.log10(d1.padj),
@@ -453,7 +517,7 @@ class RNADiffTable:
             df["names"] = self.sample_names
             df["colors"] = [colors[x] for x in self.sample_names]
             df["size"] = [10] * len(df)
-            df["condition"] = [
+            df[self.condition] = [
                 self.get_cond_from_sample(sample) for sample in self.sample_names
             ]
             fig = px.scatter_3d(
@@ -461,7 +525,7 @@ class RNADiffTable:
                 x="PC1",
                 y="PC2",
                 z="PC3",
-                color="condition",
+                color=self.condition,
                 labels={
                     "PC1": "PC1 ({}%)".format(round(100 * variance[0], 2)),
                     "PC2": "PC2 ({}%)".format(round(100 * variance[1], 2)),
@@ -519,7 +583,7 @@ class RNADiffResults:
         alpha=0.05,
         log2_fc=0,
         palette=sns.color_palette(desat=0.6),
-        group="condition",
+        condition="condition",
         annot_cols=None,
         # annot_cols=["ID", "Name", "gene_biotype"],
     ):
@@ -549,27 +613,37 @@ class RNADiffResults:
         self.dds_stats = pd.read_csv(
             self.path / "overall_dds.csv", index_col=0, sep=","
         )
+        self.condition = condition
+        self.design_df = self._get_design(
+            design_file, condition=self.condition, palette=palette
+        )
+
+        # FIXME: This block was commented because:
+        # Now RNADiffAnalysis create a sorted design file according to the count matrix
+        # Not sure that creating a design from scratch at this stage is relevant.
 
         # read different results and sort by sample name all inputs
         # TODO make this a function to be reused in RNADiffAnalysis for example
-        if design_file == None:
-            conditions = []
-            labels = self.counts_raw.columns
-            for label in labels:
-                condition = input(
-                    f"Please give use a condition name for the {label} label: "
-                )
-                conditions.append(condition)
-            df = pd.DataFrame({"label": labels, "condition": conditions})
-            df.set_index("label", inplace=True)
-            df.sort_index(inplace=True)
-            col_map = dict(zip(df.loc[:, group].unique(), palette))
-            df["group_color"] = df.loc[:, group].map(col_map)
-            self.design_df = df
-        else:
-            self.design_df = self._get_design(design_file, group=group, palette=palette)
-            self.design_df.sort_index(inplace=True)
-            self.design = RNADesign(design_file)
+        # if design_file == None:
+        #     conditions = []
+        #     labels = self.counts_raw.columns
+        #     for label in labels:
+        #         condition = input(
+        #             f"Please give use a condition name for the {label} label: "
+        #         )
+        #         conditions.append(condition)
+        #     df = pd.DataFrame({"label": labels, "condition": conditions})
+        #     df.set_index("label", inplace=True)
+        #     df.sort_index(inplace=True)
+        #     col_map = dict(zip(df.loc[:, condition].unique(), palette))
+        #     df["group_color"] = df.loc[:, condition].map(col_map)
+        #     self.design_df = df
+        # else:
+        #     self.design_df = self._get_design(
+        #         design_file, condition=condition, palette=palette
+        #     )
+        #     self.design_df.sort_index(inplace=True)
+        #     self.design = RNADesign(design_file)
 
         # optional annotation
         self.fc_attribute = fc_attribute
@@ -627,6 +701,7 @@ class RNADiffResults:
                 compa,
                 alpha=self._alpha,
                 log2_fc=self._log2_fc,
+                condition=self.condition,
                 # gff=self.annotation.annotation,
             )
             for compa in self.files
@@ -638,7 +713,7 @@ class RNADiffResults:
         """Get a properly formatted dataframe from the gff."""
 
         gff = GFF3(gff_filename)
-        df = gff.get_df()
+        df = gff.df
 
         if self.annot_cols is None:
             lol = [
@@ -727,9 +802,12 @@ class RNADiffResults:
         out_dir = Path(out_dir) / "figures"
         out_dir.mkdir(exist_ok=True, parents=True)
 
-        gene_lists_dict = self.get_gene_lists(annot_col=annot_col, Nmax=2000)
+        gene_lists_dict = self.get_gene_lists(
+            annot_col=annot_col, Nmax=2000, dropna=True
+        )
         enrichment = {}
         ontologies = {"GO:0003674": "BP", "GO:0008150": "MF", "GO:0005575": "CC"}
+        failed_enrichments = []
 
         for compa in self.comparisons:
             gene_lists = gene_lists_dict[compa]
@@ -737,22 +815,46 @@ class RNADiffResults:
             pe.compute_enrichment(ontologies=ontologies.keys(), progress=False)
 
             for direction in ["up", "down", "all"]:
+                if not pe.enrichment[direction]:
+                    logger.warning(
+                        f"No enrichment computed, so no plots computed for {compa} {direction} {ontology}"
+                    )
+                    failed_enrichments.append(
+                        {
+                            "comparison": compa,
+                            "direction": direction,
+                            "GO": "all",
+                            "reason": "no enrichment computed",
+                        }
+                    )
+                    continue
+
                 for ontology in ontologies.keys():
-                    enrichment[(compa, direction, ontology)] = pe.get_data(
-                        direction, ontology, include_negative_enrichment=False
-                    )
                     pylab.figure()
-                    try:
-                        pe.plot_go_terms(direction, ontology, compute_levels=False)
-                    except:
-                        return pe
-                    pylab.tight_layout()
-                    pylab.savefig(
-                        out_dir / f"go_{compa}_{direction}_{ontologies[ontology]}.pdf"
-                    )
-                    pylab.savefig(
-                        out_dir / f"go_{compa}_{direction}_{ontologies[ontology]}.png"
+                    enrichment_df = pe.plot_go_terms(
+                        direction, ontology, compute_levels=False
                     )
+                    if enrichment_df.empty:
+                        failed_enrichments.append(
+                            {
+                                "comparison": compa,
+                                "direction": direction,
+                                "GO": ontology,
+                                "reason": "no enrichment found",
+                            }
+                        )
+                    else:
+                        enrichment[(compa, direction, ontology)] = enrichment_df
+                        pylab.tight_layout()
+                        pylab.savefig(
+                            out_dir
+                            / f"go_{compa}_{direction}_{ontologies[ontology]}.pdf"
+                        )
+                        pe.save_chart(
+                            enrichment_df,
+                            out_dir
+                            / f"chart_{compa}_{direction}_{ontologies[ontology]}.png",
+                        )
 
             logger.info(f"Panther enrichment for {compa} DONE.")
 
@@ -762,6 +864,7 @@ class RNADiffResults:
         )
 
         self.enrichment_go = df
+        self.failed_go_enrichments = pd.DataFrame(failed_enrichments)
 
         # Export results (should be moved to enrichment.py at some point I think)
         with pd.ExcelWriter(out_dir.parent / "enrichment_go.xlsx") as writer:
@@ -781,7 +884,7 @@ class RNADiffResults:
         out_dir = Path(out_dir) / "figures"
         out_dir.mkdir(exist_ok=True, parents=True)
 
-        gene_lists_dict = self.get_gene_lists(annot_col=annot_col)
+        gene_lists_dict = self.get_gene_lists(annot_col=annot_col, dropna=True)
         enrichment = {}
 
         for compa in self.comparisons:
@@ -817,7 +920,9 @@ class RNADiffResults:
             except:
                 logger.warning("Fixme")
 
-    def get_gene_lists(self, annot_col="index", Nmax=None):  # pragma: no cover
+    def get_gene_lists(
+        self, annot_col="index", Nmax=None, dropna=False
+    ):  # pragma: no cover
 
         gene_lists_dict = {}
 
@@ -857,15 +962,28 @@ class RNADiffResults:
                 "all": all_genes,
             }
 
+            if dropna:
+                for direction in gene_lists_dict[compa]:
+                    gl = gene_lists_dict[compa][direction]
+                    if not gl:
+                        continue
+                    perc_unannotated = gl.count(None) / len(gl) * 100
+                    logger.warning(
+                        f"{compa} {direction}: Removing {perc_unannotated:.0f}% of the data for enrichment analysis due to missing identifiers in annotation."
+                    )
+                    gene_lists_dict[compa][direction] = [x for x in gl if x != None]
+
         return gene_lists_dict
 
-    def _get_design(self, design_file, group, palette):
+    def _get_design(self, design_file, condition, palette):
         """Import design from a table file and add color groups following the
-        groups defined in the column 'group' of the table file.
+        groups defined in the column 'condition' of the table file.
         """
-        df = pd.read_csv(design_file, sep=",", index_col=0)
-        col_map = dict(zip(df.loc[:, group].unique(), palette))
-        df["group_color"] = df.loc[:, group].map(col_map)
+
+        df = RNADesign(design_file).df.set_index("label")
+
+        col_map = dict(zip(df.loc[:, condition].unique(), palette))
+        df["group_color"] = df.loc[:, condition].map(col_map)
         return df
 
     def _format_plot(self, title="", xlabel="", ylabel="", rotation=0):
@@ -1049,7 +1167,7 @@ class RNADiffResults:
 
             df = pd.concat([df, self.design_df], axis=1)
             df["label"] = df.index
-            df["group_color"] = df["condition"]
+            df["group_color"] = df[self.condition]
 
             fig = px.scatter_3d(
                 df,
@@ -1201,9 +1319,9 @@ class RNADiffResults:
 
         # Convert groups into numbers for Dendrogram category
         group_conv = {
-            group: i for i, group in enumerate(self.design_df.condition.unique())
+            group: i for i, group in enumerate(self.design_df[self.condition].unique())
         }
-        d.category = self.design_df.condition.map(group_conv).to_dict()
+        d.category = self.design_df[self.condition].map(group_conv).to_dict()
         d.plot()
 
     def plot_boxplot_rawdata(self, fliersize=2, linewidth=2, rotation=0, **kwargs):
