@@ -227,19 +227,40 @@ def summary(**kwargs):
             ff = GFF3(filename)
             print("#filename: {}".format(filename))
             print("#Number of entries per genetic type:")
-            print(ff.get_df().value_counts('type').to_string())
+            print(ff.df.value_counts('type').to_string())
             print("#Number of duplicated attribute (if any) per attribute:")
             ff.get_duplicated_attributes_per_type()
 
 
+
+
+@main.command()
+@click.option("--file1", type=click.Path(),
+    default=None, required=True,
+    help="""The first input RNA-seq table to compare""")
+@click.option("--file2", type=click.Path(),
+    default=None, required=True,
+    help="""The second input RNA-seq table to compare""")
+@common_logger
+def rnaseq_compare(**kwargs):
+    """Compare 2 tables created by the 'sequana rnadiff' command"""
+    from sequana.compare import RNADiffCompare
+    c = RNADiffCompare(kwargs['file1'], kwargs['file2'])
+    c.plot_volcano_differences()
+    from pylab import savefig
+    savefig("sequana_rnaseq_compare_volcano.png", dpi=200)
+
+
+
+
 @main.command()
 @click.option("--annotation", type=click.Path(),
     default=None,
-    help="""The GFF file used to perform the annotation""")
+    help="""The annotation GFF file used to perform the feature count""")
 @click.option("--report-only", 
     is_flag=True,
     default=False,
-    help="""generate report only assuming results are already present""")
+    help="""Generate report assuming results are already present""")
 @click.option("--output-directory", type=click.Path(),
     default="rnadiff",
     help="""Output directory where are saved the results""")
@@ -247,6 +268,8 @@ def summary(**kwargs):
     default="all_features.out",
     help="""The Counts from feature counts. This should be the output of the
             sequana_rnaseq pipeline all_features.out """)
+#FIXME I think it would be better to have a single file with multiple columns
+#for alternative condition (specified using the "condition" option)
 @click.option("--design", type=click.Path(),
     default="design.csv", help="""It should have been generated by sequana_rnaseq. If
 not, it must be a comma separated file with two columns. One for the label to be
@@ -261,31 +284,62 @@ WT3,WT
 file1,cond1
 fileother,cond1
 """)
+@click.option("--condition", type=str,
+    default="condition", help="""The name of the column in design.csv to use as condition 
+for the differential analysis. Default is 'condition'""")
 @click.option("--feature-name",
     default="gene",
-    help="""The feature name compatible with your GFF. """)
+    help="""The feature name compatible with your GFF. Default is 'gene'""")
 @click.option("--attribute-name",
     default="ID",
-    help="""The attribute used as identifier. compatible with your GFF. """)
+    help="""The attribute used as identifier. compatible with your GFF. Default is 'ID'""")
 @click.option("--reference", type=click.Path(),
     default=None,
     help="""The reference to test DGE against. If provided, conditions not
-            involving the reference are ignored.""")
+            involving the reference are ignored. Otherwise all combinations are 
+            tested""")
 @click.option("--comparisons", type=click.Path(),
     default=None,
     help="""Not yet implemented. By default, all comparisons are computed""")
+
+@click.option("--cooks-cutoff", type=click.Path(),
+    default=None,
+    help="""if none, let DESeq2 choose the cutoff""")
+@click.option("--independent-filtering/--no-independent-filtering",
+    default=False,
+    help="""Do not perform independent_filtering by default. low counts may not
+have adjusted pvalues otherwise""")
+@click.option("--beta-prior/--no-beta-prior",
+    default=False,
+    help="Use beta priori or not. Default is no beta prior")
+@click.option("--fit-type",
+    default="parametric",
+    help="DESeq2 type of fit. Default is 'parametric'")
+
 @common_logger
 def rnadiff(**kwargs):
-    """Create HTML report for the differential analysis of sequana_rnaseq output.
+    """Perform RNA-seq differential analysis.
 
+    This command performs the differential analysis of gene expression. The
+    analysis is performed on feature counts generated by a RNA-seq analysis 
+    (see e.g. https://github.com/sequana/rnaseq pipeline). The analysis is
+    performed by DESeq2. A HTML report is created as well as a set of output
+    files, including summary table of the analysis.
 
-        sequana rnadiff --annotation Lepto.gff  \
-            --design design.csv --features all_features.out \
+    To perform this analysis, you will need the GFF file used during the RNA-seq
+    analysis, the feature stored altogether in a single file, an experimental 
+    design file, and the feature and attribute used during the feature count. 
+
+    Here is an example:
+
+\b
+        sequana rnadiff --annotation Lepto.gff  
+            --design design.csv --features all_features.out 
              --feature-name gene --attribute-name ID 
 
 
-
     """
+    import pandas as pd
     from sequana.featurecounts import FeatureCount
     from sequana.rnadiff import RNADiffAnalysis, RNADesign
     from sequana.modules_report.rnadiff import RNAdiffModule
@@ -303,7 +357,7 @@ def rnadiff(**kwargs):
         logger.info(f"Checking annotation file")
         from sequana import GFF3
         g = GFF3(gff) #.save_annotation_to_csv()
-        if feature not in  g.get_types():
+        if feature not in  g.features:
             logger.critical(f"{feature} not found in the GFF. Most probably a wrong feature name")
         attributes = g.get_attributes(feature)
         if attribute not in attributes:
@@ -313,23 +367,36 @@ def rnadiff(**kwargs):
         gff = None
 
     design_check = RNADesign(design, reference=reference)
-    comparisons = kwargs['comparisons']
-    if comparisons is None:
+    compa_csv = kwargs['comparisons']
+    if compa_csv:
+        compa_df = pd.read_csv(compa_csv)
+        comparisons = list(zip(compa_df["alternative"], compa_df["reference"]))
+    else:
         comparisons = design_check.comparisons
 
     if kwargs['report_only'] is False:
-        logger.info(f"Processing features counts and saving into light_counts.csv")
+        logger.info(f"Processing features counts and saving into {outdir}/light_counts.csv")
         fc = FeatureCount(kwargs['features'])
-        fc.rnadiff_df.to_csv("light_counts.csv")
+        from easydev import mkdirs
+        mkdirs(f"{outdir}")
+        fc.rnadiff_df.to_csv(f"{outdir}/light_counts.csv")
 
-        logger.info(f"Differential analysis. Saving into ./{outdir}")
-        r = RNADiffAnalysis("light_counts.csv", design,
-                condition="condition",
+        logger.info(f"Differential analysis to be saved into ./{outdir}")
+        for k in sorted(["independent_filtering", "beta_prior", 
+                        "cooks_cutoff", "fit_type", "reference"]):
+            logger.info(f"  Parameter {k} set to : {kwargs[k]}")
+        r = RNADiffAnalysis(f"{outdir}/light_counts.csv", design,
+                condition=kwargs["condition"],
                 comparisons=comparisons,
                 fc_feature=feature,
                 fc_attribute=attribute,
                 outdir=outdir,
-                gff=gff)
+                gff=gff,
+                cooks_cutoff=kwargs.get("cooks_cutoff"),
+                independent_filtering=kwargs.get("independent_filtering"),
+                beta_prior=kwargs.get("beta_prior"),
+                fit_type=kwargs.get('fit_type')
+        )
 
         logger.info(f"Saving output files into {outdir}/rnadiff.csv")
         try:
@@ -341,9 +408,9 @@ def rnadiff(**kwargs):
         else:
             logger.info(f"DGE done.")
             # cleanup if succesful
-            os.remove("rnadiff.err")
-            os.remove("rnadiff.out")
-            os.remove("rnadiff_light.R")
+            os.remove(f"{outdir}/rnadiff.err")
+            os.remove(f"{outdir}/rnadiff.out")
+            os.remove(f"{outdir}/rnadiff_light.R")
 
 
     logger.info(f"Reporting. Saving in rnadiff.html")
@@ -352,7 +419,7 @@ def rnadiff(**kwargs):
                 fc_feature=feature,
                 alpha=0.05,
                 log2_fc=0,
-                group="condition",
+                condition=kwargs["condition"],
                 annot_cols=None,
                 pattern="*vs*_degs_DESeq2.csv")
 
@@ -406,6 +473,8 @@ def biomart(**kwargs):
     help="A GFF file compatible with your salmon file")
 @click.option("-a", "--attribute", default="ID",
     help="A valid attribute to be found in the GFF file and salmon input")
+@click.option("-a", "--feature", default="gene",
+    help="A valid feature")
 def salmon(**kwargs):
     """Convert output of Salmon into a feature counts file """
     from sequana import salmon
@@ -415,8 +484,12 @@ def salmon(**kwargs):
         logger.critical("Input file does not exists ({})".format(salmon_input))
     gff = kwargs["gff"]
     attribute = kwargs['attribute']
-    s = salmon.Salmon(salmon_input)
-    s.save_feature_counts(output, gff, attribute=attribute)
+    feature = kwargs['feature']
+
+    # reads file generated by salmon and generated count file as expected by
+    # DGE. 
+    s = salmon.Salmon(salmon_input, gff)
+    s.save_feature_counts(output, feature=feature, attribute=attribute)
 
 
 @main.command()
@@ -585,6 +658,8 @@ def enrichment(**kwargs):
             logger.info(f"Computing enrichment for the {compa} case")
             logger.info(f"Found {Nup} genes up-regulated, {Ndown} down regulated ({N} in total).")
             config.output_dir = f"enrichment/{compa}"
+            try:os.mkdir("enrichment")
+            except:pass
             report = Enrichment(gene_dict, taxon, df,
                 kegg_organism=keggname, 
                 enrichment_params=params,
@@ -631,7 +706,23 @@ def taxonomy(**kwargs):
         indices = set(indices)
         print(df.loc[indices])
  
+@main.command()
+@click.argument("gff_filename", type=click.Path(exists=True))
+@common_logger
+def gff2gtf(**kwargs):
+    """Convert a GFF file into GTF
 
+    This is experimental convertion. Use with care.
+
+    """
+    filename = kwargs["gff_filename"]
+    assert filename.endswith(".gff") or filename.endswith(".gff3")
+    from sequana.gff3 import GFF3
+    g = GFF3(filename)
+    if filename.endswith(".gff"):
+        g.to_gtf(os.path.basename(filename).replace(".gff", ".gtf"))
+    elif filename.endswith(".gff3"):
+        g.to_gtf(os.path.basename(filename).replace(".gff3", ".gtf"))
 
 
 
