@@ -65,6 +65,7 @@ __all__ = ["DOTParser", "FastQFactory", "FileFactory",
             "SnakeMakeStats",
            "SequanaConfig", "modules", "pipeline_names"]
 
+
 import snakemake
 """    logger.warning("Snakemake must be installed. Available for Python3 only")
     class MockSnakeMake(object):
@@ -508,6 +509,17 @@ or open a Python shell and type::
             return self._requirements
     requirements = property(_get_requirements, doc="list of requirements")
 
+
+    def _get_required_binaries(self):
+        with open(f"{self.path}/{self.name}.rules", "r") as fout:
+            for line in fout.readlines():
+                if line.startswith('binaries'):
+                    line = line.split("=")[1].strip()
+                    binaries = line[1:-1].split(",")
+                    return binaries
+        return []
+    required_binaries = property(_get_required_binaries)
+
     def is_executable(self, verbose=False):
         """Is the module executable
 
